/*
 * Created on 15.04.2005 by Daniel Hohenberger.
 */
package de.hd42.itemlister;

import java.awt.Dimension;
import java.awt.event.*;
import java.io.IOException;
import java.util.*;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jdom.Element;

import com.Ostermiller.util.Base64;
import com.Ostermiller.util.Browser;


public class Item implements ActionListener {
	private String name = ""; //$NON-NLS-1$
	private String place = ""; //$NON-NLS-1$
	private int count = 0;
	private List<String> classes = new ArrayList<String>();
	private String itemClass = "";
	private boolean special = false;
	private boolean unique = false;
	private String uniqueString = "";
	private String comment = ""; //$NON-NLS-1$
	private String found = ""; //$NON-NLS-1$
	private boolean newItem = false;
	private String category=Strings.getString("ItemLister.Category.Standard.Name"); //$NON-NLS-1$
	private String categoryIndex=Strings.getString("ItemLister.Category.Standard.Index"); //$NON-NLS-1$
	private static final String Weapons = Strings.getString("ItemLister.Category.Weapons"); //$NON-NLS-1$
	private static final String Armor = Strings.getString("ItemLister.Category.Armor"); //$NON-NLS-1$
	private static Map<String,String> cats = new TreeMap<String,String>();
	
	
	public Item(Element e) {
		if(cats.isEmpty()) {
			Item.cats.put(Strings.getString("ItemLister.Category.Weapons.Index"),Strings.getString("ItemLister.Category.Weapons.Name"));
			Item.cats.put(Strings.getString("ItemLister.Category.Armor.Index"),Strings.getString("ItemLister.Category.Armor.Name"));
			Item.cats.put(Strings.getString("ItemLister.Category.Standard.Index"),Strings.getString("ItemLister.Category.Standard.Name"));
		}
		if(e.getName().equalsIgnoreCase(Strings.getString("ItemLister.XML.Item"))) { //$NON-NLS-1$
			this.name = e.getChildText(Strings.getString("ItemLister.XML.Name")); //$NON-NLS-1$
			this.place = e.getChildText(Strings.getString("ItemLister.XML.Place")); //$NON-NLS-1$
			if(e.getChild(Strings.getString("ItemLister.XML.Found"))!=null)this.found = e.getChildText(Strings.getString("ItemLister.XML.Found")); //$NON-NLS-1$
			this.count = Integer.parseInt(e.getChildText(Strings.getString("ItemLister.XML.Count")).replaceAll("\\D", "").trim()); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			if(e.getChild(Strings.getString("ItemLister.XML.ItemClasses"))!= null) for(Element c : (List<Element>)e.getChild(Strings.getString("ItemLister.XML.ItemClasses")).getChildren()) { //$NON-NLS-1$
				this.classes.add(c.getText());
			}
			if(this.classes.size()>0)this.itemClass=this.classes.get(0);
			if(e.getChild(Strings.getString("ItemLister.XML.Unique"))!=null) { //$NON-NLS-1$
				this.unique = true;
				//this.special = true;
				this.uniqueString = e.getChildText(Strings.getString("ItemLister.XML.Unique"));
			}
			for(String s: this.classes) {
				if(Item.Weapons.indexOf(s)>-1) {
					this.category=Strings.getString("ItemLister.Category.Weapons.Name"); //$NON-NLS-1$
					this.categoryIndex=Strings.getString("ItemLister.Category.Weapons.Index"); //$NON-NLS-1$
				}else if(Item.Armor.indexOf(s)>-1) {
					this.category=Strings.getString("ItemLister.Category.Armor.Name"); //$NON-NLS-1$
					this.categoryIndex=Strings.getString("ItemLister.Category.Armor.Index"); //$NON-NLS-1$
				}
			}
			if(e.getChild(Strings.getString("ItemLister.XML.Comment"))!=null) { //$NON-NLS-1$
				this.comment = e.getChildText(Strings.getString("ItemLister.XML.Comment"));
			}
			if(e.getAttribute(Strings.getString("ItemLister.XML.Special"))!=null){
				this.special = Boolean.valueOf(e.getAttributeValue(Strings.getString("ItemLister.XML.Special")));
			}
			
		}
		
		Item old = ItemLister.getArchivedItem(this.name);
		if(old!=null) {
			this.comment = old.getComment();
			this.itemClass = old.getItemClass();
			this.special = old.isSpecial();
			if(this.found.compareTo(old.found)>0) this.newItem = true;
		}
		
		ItemLister.archiveItem(this);
	}
	
	public static String getCatname(String index) {
		return cats.get(index);
	}
	

	/**
	 * @return Returns the itemClass.
	 */
	public String getItemClass() {
		return itemClass;
	}

	/**
	 * @param itemClass The itemClass to set.
	 */
	public void setItemClass(String itemClass) {
		this.itemClass = itemClass;
	}

	/**
	 * @return Returns the classes.
	 */
	public List<String> getClasses() {
		return classes;
	}

	/**
	 * @param classes The classes to set.
	 */
	public void setClasses(List<String> classes) {
		this.classes = classes;
	}

	/**
	 * @return Returns the count.
	 */
	public int getCount() {
		return count;
	}

	/**
	 * @param count The count to set.
	 */
	public void setCount(int count) {
		this.count = count;
	}

	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @return Returns the place.
	 */
	public String getPlace() {
		return place;
	}

	/**
	 * @param place The place to set.
	 */
	public void setPlace(String place) {
		this.place = place;
	}

	/**
	 * @return Returns the special.
	 */
	public boolean isSpecial() {
		return special;
	}

	/**
	 * @param special The special to set.
	 */
	public void setSpecial(boolean special) {
		this.special = special;
	}
	
	/**
	 * @return Returns the comment.
	 */
	public String getComment() {
		return comment;
	}

	/**
	 * @param comment The comment to set.
	 */
	public void setComment(String comment) {
		this.comment = comment;
	}

	/**
	 * @return Returns the unique.
	 */
	public boolean isUnique() {
		return unique;
	}

	/**
	 * @param unique The unique to set.
	 */
	public void setUnique(boolean unique) {
		this.unique = unique;
	}

	/**
	 * @return Returns the category.
	 */
	public String getCategory() {
		return category;
	}

	/**
	 * @param category The category to set.
	 */
	public void setCategory(String category) {
		this.category = category;
	}

	/**
	 * @return Returns the categoryIndex.
	 */
	public String getCategoryIndex() {
		return categoryIndex;
	}

	/**
	 * @param categoryIndex The categoryIndex to set.
	 */
	public void setCategoryIndex(String categoryIndex) {
		this.categoryIndex = categoryIndex;
	}

	/**
	 * @return Returns the uniqueString.
	 */
	public String getUniqueString() {
		return uniqueString;
	}

	/**
	 * @param uniqueString The uniqueString to set.
	 */
	public void setUniqueString(String uniqueString) {
		this.uniqueString = uniqueString;
	}

	/**
	 * @return Returns the found.
	 */
	public String getFound() {
		return found;
	}

	/**
	 * @param found The found to set.
	 */
	public void setFound(String found) {
		this.found = found;
	}

	/**
	 * @return Returns the newItem.
	 */
	public boolean isNewItem() {
		return newItem;
	}

	/**
	 * @param newItem The newItem to set.
	 */
	public void setNewItem(boolean newItem) {
		this.newItem = newItem;
	}

	public void actionPerformed(ActionEvent e) {
		if(Strings.getString("ItemLister.EditCommand").equals(e.getActionCommand())){ //$NON-NLS-1$
			JFrame editFrame = new JFrame(this.name);
			editFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			editFrame.addWindowListener(new java.awt.event.WindowAdapter() {
				public void windowClosing(java.awt.event.WindowEvent e) {
					ItemLister.archiveItem(Item.this);
				}
			});
			JPanel editPanel = new JPanel();
			editPanel.setLayout(new BoxLayout(editPanel, BoxLayout.Y_AXIS));
			
			JComboBox classList = new JComboBox(classes.toArray(new String[0]));
			classList.setSelectedIndex(0);
			classList.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					Item.this.setItemClass((String)((JComboBox)e.getSource()).getSelectedItem());
				}
			});
			classList.setAlignmentX(JComponent.LEFT_ALIGNMENT);
			editPanel.add(classList);
			
			SpinnerNumberModel spn = new SpinnerNumberModel(this.count, 0, 100, 1);
			JSpinner itemCount = new JSpinner(spn);
			itemCount.setMaximumSize(new Dimension(50,20));
			itemCount.addChangeListener(new ChangeListener() {public void stateChanged(ChangeEvent e) {
				Object source = e.getSource();
				if(source instanceof JSpinner){
					JSpinner sourceField = (JSpinner)source;
					SpinnerNumberModel spn = (SpinnerNumberModel)sourceField.getModel();
					int value = ((Number)spn.getNumber()).intValue();
					Item.this.setCount(value);
				}
			}});
			itemCount.setAlignmentX(JComponent.LEFT_ALIGNMENT);
			editPanel.add(itemCount);
			
			JCheckBox specialBox = new JCheckBox(Strings.getString("ItemLister.Generate.Special")); //$NON-NLS-1$
			specialBox.setSelected(this.special);
			specialBox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
				    Object source = e.getItemSelectable();
					Item.this.setSpecial(((JCheckBox)source).isSelected());
				}
			});
			specialBox.setAlignmentX(JComponent.LEFT_ALIGNMENT);
			editPanel.add(specialBox);
			
			JCheckBox newBox = new JCheckBox(Strings.getString("ItemLister.Generate.New")); //$NON-NLS-1$
			newBox.setSelected(this.newItem);
			newBox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
				    Object source = e.getItemSelectable();
					Item.this.setNewItem(((JCheckBox)source).isSelected());
				}
			});
			newBox.setAlignmentX(JComponent.LEFT_ALIGNMENT);
			editPanel.add(newBox);
			
			JTextField descriptionField = new JTextField(50);
			descriptionField.setText(this.comment);
			descriptionField.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent evt) {
			    String text = ((JTextField)evt.getSource()).getText();
				Item.this.setComment(text);
				}
			});
			descriptionField.setAlignmentX(JComponent.LEFT_ALIGNMENT);
			editPanel.add(descriptionField);
			
			editFrame.getContentPane().add(editPanel);
			editFrame.pack();
			editFrame.validate();
			editFrame.setVisible(true);
		}else if(Strings.getString("ItemLister.InfoCommand").equals(e.getActionCommand())){ //$NON-NLS-1$
			try{
				Browser.displayURL(Strings.getString("ItemLister.ItemURL") + Base64.encode(this.name)); //$NON-NLS-1$
			}catch(IOException e1){
				e1.printStackTrace();
			}
		}
	}

	public Element toXML() {
		/*<item>
			<name>Einfaches Stirnband der magischen Markierung</name>
			<place>Kopf</place>
			<count>1</count>
			<found>2005-04-20</found>
			<itemclasses>
				<name>Magische Gegenstnde</name>
			</itemclasses>
			<unique>Unique fr Held</unique>
		</item>*/
		Element item = new Element(Strings.getString("ItemLister.XML.Item"));
		Element name = new Element(Strings.getString("ItemLister.XML.Name"));
		name.setText(this.name);
		item.addContent(name);
		Element count = new Element(Strings.getString("ItemLister.XML.Count"));
		count.setText(this.count+"");
		item.addContent(count);
		Element found = new Element(Strings.getString("ItemLister.XML.Found"));
		found.setText(this.found+"");
		item.addContent(found);
		Element itemclasses = new Element(Strings.getString("ItemLister.XML.ItemClasses"));
			Element cn = new Element(Strings.getString("ItemLister.XML.Name"));
			cn.setText(this.itemClass);
			itemclasses.addContent(cn);
		item.addContent(itemclasses);
		if(this.unique) {
			Element unique = new Element(Strings.getString("ItemLister.XML.Unique"));
			unique.setText(this.uniqueString);
			item.addContent(unique);
		}
		Element comment = new Element(Strings.getString("ItemLister.XML.Comment"));
		comment.setText(this.comment);
		item.addContent(comment);
		if(this.special) {
			item.setAttribute(Strings.getString("ItemLister.XML.Special"), Boolean.toString(this.special));
		}
		
		return item;
	}
}
