/*
 * Created on 03.02.2005 by Daniel Hohenberger.
 */
package de.hd42.itemlister;

import java.awt.*;
import java.awt.event.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.*;
import java.nio.charset.Charset;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jdom.*;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import com.Ostermiller.util.Browser;

/**
 * @author Daniel Hohenberger
 */
public class ItemLister extends JFrame implements ActionListener {

	private static final long serialVersionUID = 3257568386639802934L;
	private JPanel listPanel;
	private MakeFrame makeFrame;
	private Hashtable<JSpinner, Item> fields = new Hashtable<JSpinner, Item>();
	private Hashtable<JCheckBox, Item> boxes = new Hashtable<JCheckBox, Item>();
	public static final String newString = Strings.getString("ItemLister.NewItem"); //$NON-NLS-1$
	public static final String PCN = Strings.getString("ItemLister.PCN"); //$NON-NLS-1$
	public static final String CPN = Strings.getString("ItemLister.CPN"); //$NON-NLS-1$
	public static final String PN = Strings.getString("ItemLister.PN"); //$NON-NLS-1$
	public static final String CN = Strings.getString("ItemLister.CN"); //$NON-NLS-1$
	private static String sorting = ItemLister.PCN;
	private static boolean useCats = true;
	private static Map<String,Item> archiv = new TreeMap<String,Item>();
	
	public static void main(String[] args) {
		ItemLister il = new ItemLister();
		Browser.init();
		il.setVisible(true);
	}

	public ItemLister() {
		super(Strings.getString("ItemLister.Title")); //$NON-NLS-1$
		try{
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}catch(Exception e){}
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.addWindowListener(new java.awt.event.WindowAdapter() {
			public void windowClosing(java.awt.event.WindowEvent e) {
				ItemLister.saveArchive();
				System.exit(0);
			}
		});
		this.setSize(1024, 736);

		JPanel mainPanel = new JPanel();
		listPanel = new JPanel();
		listPanel.setLayout(new BoxLayout(listPanel, BoxLayout.Y_AXIS));
		JScrollPane listScroll = new JScrollPane(listPanel);
		listScroll.setPreferredSize(new Dimension(1000, 640));
		mainPanel.add(listScroll, BorderLayout.NORTH);
		JButton openButton = new JButton(Strings.getString("ItemLister.LoadButton")); //$NON-NLS-1$
		openButton.setActionCommand(Strings.getString("ItemLister.LoadCommand")); //$NON-NLS-1$
		openButton.addActionListener(this);
		mainPanel.add(openButton, BorderLayout.SOUTH);
		JButton makeButton = new JButton(Strings.getString("ItemLister.GenerateButton")); //$NON-NLS-1$
		makeButton.setActionCommand(Strings.getString("ItemLister.GenerateCommand")); //$NON-NLS-1$
		makeButton.addActionListener(this);
		mainPanel.add(makeButton, BorderLayout.SOUTH);
		JButton archiveButton = new JButton(Strings.getString("ItemLister.ArchivButton")); //$NON-NLS-1$
		archiveButton.setActionCommand(Strings.getString("ItemLister.ArchivCommand")); //$NON-NLS-1$
		archiveButton.addActionListener(this);
		mainPanel.add(archiveButton, BorderLayout.SOUTH);

		this.getContentPane().add(mainPanel, BorderLayout.CENTER);
		
		File archiveFile = new File(Strings.getString("ItemLister.Archiv"));
		if(!archiveFile.exists()) {
			try{
				archiveFile.createNewFile();
			}catch(IOException e1){
				e1.printStackTrace();
			}
		}
		List<Item> items = this.readFile(archiveFile);
		for(Item item: items) {
			ItemLister.archiv.put(item.getName(), item);
		}
	}

	public void actionPerformed(ActionEvent e) {
		if(Strings.getString("ItemLister.LoadCommand").equals(e.getActionCommand())){ //$NON-NLS-1$
			File inputfile = ItemLister
					.chooseFile(Strings.getString("ItemLister.LoadButton"), this); //$NON-NLS-1$
			this.makePanel(this.readFile(inputfile));
		}else if(Strings.getString("ItemLister.GenerateCommand").equals(e.getActionCommand())){ //$NON-NLS-1$
			this.showOutputFrame();
		}else if(Strings.getString("ItemLister.ArchivCommand").equals(e.getActionCommand())){ //$NON-NLS-1$
			ItemLister.saveArchive();
		}
	}

	private void saveToFile(String text, File outputfile) {
		try{
			BufferedWriter outStream = new BufferedWriter(new FileWriter(outputfile));
			outStream.write(text);
			outStream.close();
		}catch(IOException e){
			e.printStackTrace();
		}
	}

	public static File chooseFile(String confirm, Frame frame) {
		JFileChooser chooser = new JFileChooser();
		chooser.showDialog(frame, confirm);
		return chooser.getSelectedFile();
	}

	private List<Item> readFile(File f) {
		SAXBuilder builder = new SAXBuilder();
		if(f!=null && f.exists()) {
			try{
				Document doc = builder.build(f);
				 return this.buildItemList(doc);
			}catch(JDOMException e){
				System.out.println(f + " is not well-formed."); //$NON-NLS-1$
				System.out.println(e.getMessage());
			}catch(IOException e){
				System.out.println("Could not read " + f); //$NON-NLS-1$
				System.out.println(" because " + e.getMessage()); //$NON-NLS-1$
			}
		}
		return new ArrayList<Item>();
	}

	private List<Item> buildItemList(Document doc) {
		List<Item> items = new ArrayList<Item>();
		List<Element> itemElements = doc.getRootElement().getChildren(
				Strings.getString("ItemLister.XML.Item")); //$NON-NLS-1$
		for(Element e : itemElements){
			items.add(new Item(e));
		}
		return items;
	}
	
	private void makePanel(List<Item> items) {
		for(Item item : items){
			JPanel itemPanel = new JPanel();
			itemPanel.setLayout(new BoxLayout(itemPanel, BoxLayout.X_AXIS));
			itemPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));

			JCheckBox newBox = new JCheckBox(Strings.getString("ItemLister.Generate.New")); //$NON-NLS-1$
			newBox.setSelected(item.isNewItem());
			boxes.put(newBox, item);
			newBox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
				    Object source = e.getItemSelectable();
					boxes.get(((JCheckBox)source)).setNewItem(((JCheckBox)source).isSelected());
				}
			});
			newBox.setAlignmentX(JComponent.LEFT_ALIGNMENT);
			itemPanel.add(newBox);
			
			JCheckBox specialBox = new JCheckBox(Strings.getString("ItemLister.Generate.Special.short")); //$NON-NLS-1$
			specialBox.setSelected(item.isSpecial());
			boxes.put(specialBox, item);
			specialBox.addItemListener(new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
				    Object source = e.getItemSelectable();
					boxes.get(((JCheckBox)source)).setSpecial(((JCheckBox)source).isSelected());
				}
			});
			specialBox.setAlignmentX(JComponent.LEFT_ALIGNMENT);
			itemPanel.add(specialBox);
			
			SpinnerNumberModel spn = new SpinnerNumberModel(item.getCount(), 0, 100, 1);
			JSpinner itemCount = new JSpinner(spn);
			itemCount.setMaximumSize(new Dimension(50, 20));
			fields.put(itemCount, item);
			itemCount.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					Object source = e.getSource();
					if(source instanceof JSpinner){
						JSpinner sourceField = (JSpinner)source;
						SpinnerNumberModel spn = (SpinnerNumberModel)sourceField.getModel();
						int value = ((Number)spn.getNumber()).intValue();
						fields.get(sourceField).setCount(value);
					}
				}
			});
			itemPanel.add(itemCount);

			JLabel itemLabel = new JLabel(item.getName()
					+ " - " + item.getItemClass() + " - " + item.getPlace()); //$NON-NLS-1$ //$NON-NLS-2$
			itemLabel.setBorder(BorderFactory.createEmptyBorder(0, 2, 0, 6));
			itemPanel.add(itemLabel);

			itemPanel.add(Box.createHorizontalGlue());

			JButton edit = new JButton(Strings.getString("ItemLister.EditButton")); //$NON-NLS-1$
			edit.setActionCommand(Strings.getString("ItemLister.EditCommand")); //$NON-NLS-1$
			edit.addActionListener(item);
			itemPanel.add(edit);

			/*JButton info = new JButton(Strings.getString("ItemLister.InfoButton")); //$NON-NLS-1$
			info.setActionCommand(Strings.getString("ItemLister.InfoCommand")); //$NON-NLS-1$
			info.addActionListener(item);
			itemPanel.add(info);*/

			itemPanel.setAlignmentX(LEFT_ALIGNMENT);
			itemPanel.setBackground(Color.WHITE);
			listPanel.add(itemPanel);
		}
		this.validate();
		this.repaint();
	}

	private void showOutputFrame() {
		this.makeFrame = new MakeFrame(this);
	}

public String createOutput() {
		Map<String,Map<String,Map<String,Map<String,Item>>>> list = new TreeMap<String,Map<String,Map<String,Map<String,Item>>>>();
		Map<String,Item> specials = new TreeMap<String,Item>();
		String cat0 = " ";
		for(Item i : fields.values()) {
			if(i.isSpecial()) {
				specials.put(i.getName(), i);
			}else {
				if(ItemLister.useCats) {
					cat0 = i.getCategoryIndex();
				}
				String cat1 = i.getPlace();
				String cat2 = i.getItemClass();
				String cat3 = i.getName();
				if(ItemLister.sorting.equals(ItemLister.CPN)) {
					cat1 = i.getItemClass();
					cat2 = i.getPlace();
				}else if(ItemLister.sorting.equals(ItemLister.PN)) {
					cat1 = Strings.getString("ItemLister.SimpleItems"); //$NON-NLS-1$
					cat2 = i.getPlace();
				}else if(ItemLister.sorting.equals(ItemLister.CN)) {
					cat1 = Strings.getString("ItemLister.SimpleItems"); //$NON-NLS-1$
					cat2 = i.getItemClass();
				}
				if(!list.containsKey(cat0)) {
					list.put(cat0, new TreeMap<String,Map<String,Map<String,Item>>>());
				}
				Map<String,Map<String,Map<String,Item>>> m0 = list.get(cat0);
				if(!m0.containsKey(cat1)) {
					m0.put(cat1, new TreeMap<String,Map<String,Item>>());
				}
				Map<String,Map<String,Item>> m1 = m0.get(cat1);
				if(!m1.containsKey(cat2)) {
					m1.put(cat2, new TreeMap<String,Item>());
				}
				Map<String,Item> m2 = m1.get(cat2);
				m2.put(cat3, i);
			}
		}
		
		StringBuffer output = new StringBuffer();

		output.append(Strings.getString("ItemLister.Out.Intro")); //$NON-NLS-1$
		if(specials.size()>0) {
			output.append(Strings.getString("ItemLister.Out.SpecialItems")); //$NON-NLS-1$
			output.append(Strings.getString("ItemLister.Out.Special.List.Pre")); //$NON-NLS-1$
			for(Map.Entry<String,Item> e3: specials.entrySet()) {
				Item i = e3.getValue();
				output.append(Strings.getString("ItemLister.Out.ListItem") ); //$NON-NLS-1$
				if(i.isUnique()) {
					output.append(Strings.getString("ItemLister.Out.Unique.Pre")); //$NON-NLS-1$
				}
				output.append(Strings.getString("ItemLister.Out.Special.Item.Pre") + i.getName() + Strings.getString("ItemLister.Out.Special.Item.Post") ); //$NON-NLS-1$ //$NON-NLS-2$
				if(i.isUnique()) {
					output.append(Strings.getString("ItemLister.Out.Unique.Post")); //$NON-NLS-1$
					output.append(Strings.getString("ItemLister.Out.Unique.Comment.Pre")); //$NON-NLS-1$
					output.append(i.getUniqueString());
					output.append(Strings.getString("ItemLister.Out.Unique.Comment.Post")); //$NON-NLS-1$
				}
				if(i.getCount()>1) {
					output.append(Strings.getString("ItemLister.Out.Count.Pre")+i.getCount()+Strings.getString("ItemLister.Out.Count.Post")); //$NON-NLS-1$ //$NON-NLS-2$
				}
				if(i.getComment().length()>0) {
					output.append(Strings.getString("ItemLister.Out.Comment.Pre")+i.getComment()+Strings.getString("ItemLister.Out.Comment.Post")); //$NON-NLS-1$ //$NON-NLS-2$
				}
				if(i.isNewItem()) {
					output.append(Strings.getString("ItemLister.Out.NewItem"));
				}
				output.append(Strings.getString("ItemLister.Out.List.EntryEnd")); //$NON-NLS-1$
			}
			output.append(Strings.getString("ItemLister.Out.Special.List.Post")); //$NON-NLS-1$
		}

		for(Map.Entry<String,Map<String,Map<String,Map<String,Item>>>> e0: list.entrySet()) {
			if(Item.getCatname(e0.getKey())!=null)if(Item.getCatname(e0.getKey()).length()>0)output.append(Strings.getString("ItemLister.Out.Level0.Pre") + Item.getCatname(e0.getKey()) + Strings.getString("ItemLister.Out.Level0.Post")); //$NON-NLS-1$ //$NON-NLS-2$
			for(Map.Entry<String,Map<String,Map<String,Item>>> e1: e0.getValue().entrySet()) {
				output.append(Strings.getString("ItemLister.Out.Level1.Pre") + e1.getKey() + Strings.getString("ItemLister.Out.Level1.Post")); //$NON-NLS-1$ //$NON-NLS-2$
				for(Map.Entry<String,Map<String,Item>> e2: e1.getValue().entrySet()) {
					output.append(Strings.getString("ItemLister.Out.Level2.Pre") + e2.getKey() + Strings.getString("ItemLister.Out.Level2.Post")); //$NON-NLS-1$ //$NON-NLS-2$
					output.append(Strings.getString("ItemLister.Out.List.Pre")); //$NON-NLS-1$
					for(Map.Entry<String,Item> e3: e2.getValue().entrySet()) {
						Item i = e3.getValue();
						output.append(Strings.getString("ItemLister.Out.ListItem") ); //$NON-NLS-1$
						if(i.isUnique()) {
							output.append(Strings.getString("ItemLister.Out.Unique.Pre")); //$NON-NLS-1$
						}
						output.append(Strings.getString("ItemLister.Out.Item.Pre") + i.getName() + Strings.getString("ItemLister.Out.Item.Post") ); //$NON-NLS-1$ //$NON-NLS-2$
						if(i.isUnique()) {
							output.append(Strings.getString("ItemLister.Out.Unique.Post")); //$NON-NLS-1$
							output.append(Strings.getString("ItemLister.Out.Unique.Comment.Pre")); //$NON-NLS-1$
							output.append(i.getUniqueString());
							output.append(Strings.getString("ItemLister.Out.Unique.Comment.Post")); //$NON-NLS-1$
						}
						if(i.getCount()>1) {
							output.append(Strings.getString("ItemLister.Out.Count.Pre")+i.getCount()+Strings.getString("ItemLister.Out.Count.Post")); //$NON-NLS-1$ //$NON-NLS-2$
						}
						if(i.getComment().length()>0) {
							output.append(Strings.getString("ItemLister.Out.Comment.Pre")+i.getComment()+Strings.getString("ItemLister.Out.Comment.Post")); //$NON-NLS-1$ //$NON-NLS-2$
						}
						if(i.isNewItem()) {
							output.append(Strings.getString("ItemLister.Out.NewItem"));
						}
						output.append(Strings.getString("ItemLister.Out.List.EntryEnd")); //$NON-NLS-1$
					}
					output.append(Strings.getString("ItemLister.Out.List.Post")); //$NON-NLS-1$
				}
			}
		}
		return output.toString();
	}

	public void saveOutput(File outputfile, String output) {
		if(outputfile!=null) {
			try{
				BufferedWriter outStream = new BufferedWriter(new FileWriter(outputfile));
				outStream.write(output);
				outStream.close();
			}catch(IOException e){
				e.printStackTrace();
			}
			makeFrame.dispose();
		}
	}


	public static Item getArchivedItem(String name) {
		return ItemLister.archiv.get(name);
	}
	
	public static void archiveItem(Item item) {
		ItemLister.archiv.put(item.getName(), item);
	}
	
	private static void saveArchive() {
		Element root = new Element(Strings.getString("ItemLister.XML.Storage"));
		for(Item item: ItemLister.archiv.values()) {
			root.addContent(item.toXML());
		}
		Document doc = new Document(root);
		Format format = Format.getPrettyFormat();
		format.setEncoding("UTF-8");
		XMLOutputter outputter = new XMLOutputter(format);
	    try {
			FileOutputStream fos = new FileOutputStream(Strings.getString("ItemLister.Archiv"));
			OutputStreamWriter osw = new OutputStreamWriter(fos, Charset.forName("UTF-8"));
	      outputter.output(doc, new BufferedWriter(osw));       
	    }
	    catch (IOException e) {
	      System.err.println(e);
	    }
	}

	/**
	 * @return Returns the sorting.
	 */
	public static String getSorting() {
		return sorting;
	}

	/**
	 * @param sorting The sorting to set.
	 */
	public static void setSorting(String sorting) {
		ItemLister.sorting = sorting;
	}

	/**
	 * @return Returns the useCats.
	 */
	public static boolean isUseCats() {
		return useCats;
	}

	/**
	 * @param useCats The useCats to set.
	 */
	public static void setUseCats(boolean useCats) {
		ItemLister.useCats = useCats;
	}
}
